@@ -88,6 +88,9 @@ class CaselessDict(OrderedDict):
     def __eq__(self, other):
         return self is other or dict(self.items()) == dict(other.items())
 
+    def __ne__(self, other):
+        return not self == other
+
     # A list of keys that must appear first in sorted_keys and sorted_items;
     # must be uppercase.
     canonical_order = None
@@ -103,3 +106,6 @@ class CaselessDict(OrderedDict):
         Items not specified in canonical_order will appear at the end.
         """
         return canonsort_items(self, self.canonical_order)
+
+
+__all__ = ["canonsort_keys", "canonsort_items", "CaselessDict"]
