@@ -1,12 +1,11 @@
-# -*- coding: utf-8 -*-
-"""This module parses and generates contentlines as defined in RFC 2445
+"""This module parses and generates contentlines as defined in RFC 5545
 (iCalendar), but will probably work for other MIME types with similar syntax.
 Eg. RFC 2426 (vCard)
 
 It is stupid in the sense that it treats the content purely as strings. No type
 conversion is attempted.
 """
-from icalendar import compat
+
 from icalendar.caselessdict import CaselessDict
 from icalendar.parser_tools import DEFAULT_ENCODING
 from icalendar.parser_tools import SEQUENCE_TYPES
@@ -18,7 +17,7 @@ import re
 def escape_char(text):
     """Format value according to iCalendar TEXT escaping rules.
     """
-    assert isinstance(text, (compat.unicode_type, compat.bytes_type))
+    assert isinstance(text, (str, bytes))
     # NOTE: ORDER MATTERS!
     return text.replace(r'\N', '\n')\
                .replace('\\', '\\\\')\
@@ -29,38 +28,25 @@ def escape_char(text):
 
 
 def unescape_char(text):
-    assert isinstance(text, (compat.unicode_type, compat.bytes_type))
+    assert isinstance(text, (str, bytes))
     # NOTE: ORDER MATTERS!
-    if isinstance(text, compat.unicode_type):
-        return text.replace(u'\\N', u'\\n')\
-                   .replace(u'\r\n', u'\n')\
-                   .replace(u'\\n', u'\n')\
-                   .replace(u'\\,', u',')\
-                   .replace(u'\\;', u';')\
-                   .replace(u'\\\\', u'\\')
-    elif isinstance(text, compat.bytes_type):
-        return text.replace(b'\N', b'\n')\
+    if isinstance(text, str):
+        return text.replace('\\N', '\\n')\
+                   .replace('\r\n', '\n')\
+                   .replace('\\n', '\n')\
+                   .replace('\\,', ',')\
+                   .replace('\\;', ';')\
+                   .replace('\\\\', '\\')
+    elif isinstance(text, bytes):
+        return text.replace(b'\\N', b'\\n')\
                    .replace(b'\r\n', b'\n')\
-                   .replace(b'\n', b'\n')\
-                   .replace(b'\,', b',')\
-                   .replace(b'\;', b';')\
+                   .replace(b'\\n', b'\n')\
+                   .replace(b'\\,', b',')\
+                   .replace(b'\\;', b';')\
                    .replace(b'\\\\', b'\\')
 
 
-def tzid_from_dt(dt):
-    tzid = None
-    if hasattr(dt.tzinfo, 'zone'):
-        tzid = dt.tzinfo.zone  # pytz implementation
-    elif hasattr(dt.tzinfo, 'tzname'):
-        try:
-            tzid = dt.tzinfo.tzname(dt)  # dateutil implementation
-        except AttributeError:
-            # No tzid available
-            pass
-    return tzid
-
-
-def foldline(line, limit=75, fold_sep=u'\r\n '):
+def foldline(line, limit=75, fold_sep='\r\n '):
     """Make a string folded as defined in RFC5545
     Lines of text SHOULD NOT be longer than 75 octets, excluding the line
     break.  Long content lines SHOULD be split into a multiple line
@@ -69,20 +55,30 @@ def foldline(line, limit=75, fold_sep=u'\r\n '):
     immediately followed by a single linear white-space character (i.e.,
     SPACE or HTAB).
     """
-    assert isinstance(line, compat.unicode_type)
-    assert u'\n' not in line
+    assert isinstance(line, str)
+    assert '\n' not in line
 
-    ret_line = u''
+    # Use a fast and simple variant for the common case that line is all ASCII.
+    try:
+        line.encode('ascii')
+    except (UnicodeEncodeError, UnicodeDecodeError):
+        pass
+    else:
+        return fold_sep.join(
+            line[i:i + limit - 1] for i in range(0, len(line), limit - 1)
+        )
+
+    ret_chars = []
     byte_count = 0
     for char in line:
         char_byte_len = len(char.encode(DEFAULT_ENCODING))
         byte_count += char_byte_len
         if byte_count >= limit:
-            ret_line += fold_sep
+            ret_chars.append(fold_sep)
             byte_count = char_byte_len
-        ret_line += char
+        ret_chars.append(char)
 
-    return ret_line
+    return ''.join(ret_chars)
 
 
 #################################################################
@@ -93,19 +89,22 @@ def param_value(value):
     """
     if isinstance(value, SEQUENCE_TYPES):
         return q_join(value)
-    return dquote(value)
+    elif isinstance(value, str):
+        return dquote(value)
+    else:
+        return dquote(value.to_ical().decode(DEFAULT_ENCODING))
 
 
 # Could be improved
 
 # [\w-] because of the iCalendar RFC
-# \. because of the vCard RFC
-NAME = re.compile('[\w\.-]+')
+# . because of the vCard RFC
+NAME = re.compile(r'[\w.-]+')
 
 UNSAFE_CHAR = re.compile('[\x00-\x08\x0a-\x1f\x7F",:;]')
 QUNSAFE_CHAR = re.compile('[\x00-\x08\x0a-\x1f\x7F"]')
 FOLD = re.compile(b'(\r?\n)+[ \t]')
-uFOLD = re.compile(u'(\r?\n)+[ \t]')
+uFOLD = re.compile('(\r?\n)+[ \t]')
 NEWLINE = re.compile(r'\r?\n')
 
 
@@ -134,27 +133,32 @@ def dquote(val):
     # so replace it with a single-quote character
     val = val.replace('"', "'")
     if QUOTABLE.search(val):
-        return '"%s"' % val
+        return f'"{val}"'
     return val
 
 
 # parsing helper
-def q_split(st, sep=','):
+def q_split(st, sep=',', maxsplit=-1):
     """Splits a string on char, taking double (q)uotes into considderation.
     """
+    if maxsplit == 0:
+        return [st]
+
     result = []
     cursor = 0
     length = len(st)
     inquote = 0
-    for i in range(length):
-        ch = st[i]
+    splits = 0
+    for i, ch in enumerate(st):
         if ch == '"':
             inquote = not inquote
         if not inquote and ch == sep:
             result.append(st[cursor:i])
             cursor = i + 1
-        if i + 1 == length:
+            splits += 1
+        if i + 1 == length or splits == maxsplit:
             result.append(st[cursor:])
+            break
     return result
 
 
@@ -170,7 +174,7 @@ class Parameters(CaselessDict):
     """
 
     def params(self):
-        """In rfc2445 keys are called parameters, so this is to be consitent
+        """In RFC 5545 keys are called parameters, so this is to be consitent
         with the naming conventions.
         """
         return self.keys()
@@ -202,7 +206,7 @@ class Parameters(CaselessDict):
 
         for key, value in items:
             value = param_value(value)
-            if isinstance(value, compat.unicode_type):
+            if isinstance(value, str):
                 value = value.encode(DEFAULT_ENCODING)
             # CaselessDict keys are always unicode
             key = key.upper().encode(DEFAULT_ENCODING)
@@ -217,7 +221,7 @@ class Parameters(CaselessDict):
         result = cls()
         for param in q_split(st, ';'):
             try:
-                key, val = q_split(param, '=')
+                key, val = q_split(param, '=', maxsplit=1)
                 validate_token(key)
                 # Property parameter values that are not in quoted
                 # strings are case insensitive.
@@ -241,13 +245,13 @@ class Parameters(CaselessDict):
                     else:
                         result[key] = vals
             except ValueError as exc:
-                raise ValueError('%r is not a valid parameter string: %s'
-                                 % (param, exc))
+                raise ValueError(
+                    f'{param!r} is not a valid parameter string: {exc}')
         return result
 
 
 def escape_string(val):
-    # '%{:02X}'.format(i)
+    # f'{i:02X}'
     return val.replace(r'\,', '%2C').replace(r'\:', '%3A')\
               .replace(r'\;', '%3B').replace(r'\\', '%5C')
 
@@ -267,15 +271,15 @@ def unescape_list_or_string(val):
 #########################################
 # parsing and generation of content lines
 
-class Contentline(compat.unicode_type):
+class Contentline(str):
     """A content line is basically a string that can be folded and parsed into
     parts.
     """
     def __new__(cls, value, strict=False, encoding=DEFAULT_ENCODING):
         value = to_unicode(value, encoding=encoding)
-        assert u'\n' not in value, ('Content line can not contain unescaped '
-                                    'new line characters.')
-        self = super(Contentline, cls).__new__(cls, value)
+        assert '\n' not in value, ('Content line can not contain unescaped '
+                                   'new line characters.')
+        self = super().__new__(cls, value)
         self.strict = strict
         return self
 
@@ -297,8 +301,8 @@ class Contentline(compat.unicode_type):
         values = to_unicode(values)
         if params:
             params = to_unicode(params.to_ical(sorted=sorted))
-            return cls(u'%s;%s:%s' % (name, params, values))
-        return cls(u'%s:%s' % (name, values))
+            return cls(f'{name};{params}:{values}')
+        return cls(f'{name}:{values}')
 
     def parts(self):
         """Split the content line up into (name, parameters, values) parts.
@@ -320,21 +324,21 @@ class Contentline(compat.unicode_type):
             if not name:
                 raise ValueError('Key name is required')
             validate_token(name)
+            if not value_split:
+                value_split = i + 1
             if not name_split or name_split + 1 == value_split:
                 raise ValueError('Invalid content line')
             params = Parameters.from_ical(st[name_split + 1: value_split],
                                           strict=self.strict)
             params = Parameters(
                 (unescape_string(key), unescape_list_or_string(value))
-                for key, value in compat.iteritems(params)
+                for key, value in iter(params.items())
             )
             values = unescape_string(st[value_split + 1:])
             return (name, params, values)
         except ValueError as exc:
             raise ValueError(
-                u"Content line could not be parsed into parts: %r: %s"
-                % (self, exc)
-            )
+                f"Content line could not be parsed into parts: '{self}': {exc}")
 
     @classmethod
     def from_ical(cls, ical, strict=False):
@@ -356,6 +360,7 @@ class Contentlines(list):
     Then this should be efficient. for Huge files, an iterator should probably
     be used instead.
     """
+
     def to_ical(self):
         """Simply join self.
         """
@@ -370,12 +375,20 @@ class Contentlines(list):
             # a fold is carriage return followed by either a space or a tab
             unfolded = uFOLD.sub('', st)
             lines = cls(Contentline(line) for
-                        line in unfolded.splitlines() if line)
+                        line in NEWLINE.split(unfolded) if line)
             lines.append('')  # '\r\n' at the end of every content line
             return lines
-        except:
+        except Exception:
             raise ValueError('Expected StringType with content lines')
 
 
 # XXX: what kind of hack is this? import depends to be at end
 from icalendar.prop import vText
+
+
+__all__ = ["Contentline", "Contentlines", "FOLD", "NAME", "NEWLINE",
+           "Parameters", "QUNSAFE_CHAR", "QUOTABLE", "UNSAFE_CHAR", "dquote",
+           "escape_char", "escape_string", "foldline", "param_value", "q_join",
+           "q_split", "uFOLD", "unescape_char",
+           "unescape_list_or_string", "unescape_string", "validate_param_value",
+           "validate_token"]
