@@ -1,8 +1,8 @@
-# -*- coding: utf-8 -*-
 """This module contains the parser/generators (or coders/encoders if you
 prefer) for the classes/datatypes that are used in iCalendar:
 
 ###########################################################################
+
 # This module defines these property value data types and property parameters
 
 4.2 Defined property parameters are:
@@ -20,7 +20,7 @@ prefer) for the classes/datatypes that are used in iCalendar:
 
 iCalendar properties have values. The values are strongly typed. This module
 defines these types, calling val.to_ical() on them will render them as defined
-in rfc2445.
+in rfc5545.
 
 If you pass any of these classes a Python primitive, you will have an object
 that can render itself as iCalendar formatted date.
@@ -40,100 +40,46 @@ from datetime import datetime
 from datetime import time
 from datetime import timedelta
 from datetime import tzinfo
-
-try:
-    from dateutil.tz import tzutc
-except ImportError:
-    tzutc = None
-
-from icalendar import compat
 from icalendar.caselessdict import CaselessDict
 from icalendar.parser import Parameters
 from icalendar.parser import escape_char
-from icalendar.parser import tzid_from_dt
 from icalendar.parser import unescape_char
-from icalendar.parser_tools import DEFAULT_ENCODING
-from icalendar.parser_tools import SEQUENCE_TYPES
-from icalendar.parser_tools import to_unicode
-from icalendar.timezone_cache import _timezone_cache
-from icalendar.windows_to_olson import WINDOWS_TO_OLSON
+from icalendar.parser_tools import (
+    DEFAULT_ENCODING, SEQUENCE_TYPES, to_unicode, from_unicode, ICAL_TYPE
+)
 
 import base64
 import binascii
-import pytz
+from .timezone import tzp
 import re
 import time as _time
 
+from typing import Optional, Union
+from enum import Enum, auto
 
-DATE_PART = r'(\d+)D'
-TIME_PART = r'T(?:(\d+)H)?(?:(\d+)M)?(?:(\d+)S)?'
-DATETIME_PART = '(?:%s)?(?:%s)?' % (DATE_PART, TIME_PART)
-WEEKS_PART = r'(\d+)W'
-DURATION_REGEX = re.compile(r'([-+]?)P(?:%s|%s)$'
-                            % (WEEKS_PART, DATETIME_PART))
-WEEKDAY_RULE = re.compile(r'(?P<signal>[+-]?)(?P<relative>[\d]?)'
+
+DURATION_REGEX = re.compile(r'([-+]?)P(?:(\d+)W)?(?:(\d+)D)?'
+                            r'(?:T(?:(\d+)H)?(?:(\d+)M)?(?:(\d+)S)?)?$')
+
+WEEKDAY_RULE = re.compile(r'(?P<signal>[+-]?)(?P<relative>[\d]{0,2})'
                           r'(?P<weekday>[\w]{2})$')
 
 
-####################################################
-# handy tzinfo classes you can use.
-#
-
-ZERO = timedelta(0)
-HOUR = timedelta(hours=1)
-STDOFFSET = timedelta(seconds=-_time.timezone)
-if _time.daylight:
-    DSTOFFSET = timedelta(seconds=-_time.altzone)
-else:
-    DSTOFFSET = STDOFFSET
-DSTDIFF = DSTOFFSET - STDOFFSET
+def tzid_from_dt(dt: datetime) -> Optional[str]:
+    """Retrieve the timezone id from the datetime object."""
+    tzid = None
+    if hasattr(dt.tzinfo, 'zone'):
+        tzid = dt.tzinfo.zone  # pytz implementation
+    elif hasattr(dt.tzinfo, 'key'):
+        tzid = dt.tzinfo.key  # ZoneInfo implementation
+    elif hasattr(dt.tzinfo, 'tzname'):
+        # dateutil implementation, but this is broken
+        # See https://github.com/collective/icalendar/issues/333 for details
+        tzid = dt.tzinfo.tzname(dt)
+    return tzid
 
 
-class FixedOffset(tzinfo):
-    """Fixed offset in minutes east from UTC.
-    """
-    def __init__(self, offset, name):
-        self.__offset = timedelta(minutes=offset)
-        self.__name = name
-
-    def utcoffset(self, dt):
-        return self.__offset
-
-    def tzname(self, dt):
-        return self.__name
-
-    def dst(self, dt):
-        return ZERO
-
-
-class LocalTimezone(tzinfo):
-    """Timezone of the machine where the code is running.
-    """
-    def utcoffset(self, dt):
-        if self._isdst(dt):
-            return DSTOFFSET
-        else:
-            return STDOFFSET
-
-    def dst(self, dt):
-        if self._isdst(dt):
-            return DSTDIFF
-        else:
-            return ZERO
-
-    def tzname(self, dt):
-        return _time.tzname[self._isdst(dt)]
-
-    def _isdst(self, dt):
-        tt = (dt.year, dt.month, dt.day,
-              dt.hour, dt.minute, dt.second,
-              dt.weekday(), 0, -1)
-        stamp = _time.mktime(tt)
-        tt = _time.localtime(stamp)
-        return tt.tm_isdst > 0
-
-
-class vBinary(object):
+class vBinary:
     """Binary property values are base 64 encoded.
     """
 
@@ -142,7 +88,7 @@ class vBinary(object):
         self.params = Parameters(encoding='BASE64', value="BINARY")
 
     def __repr__(self):
-        return "vBinary('%s')" % self.to_ical()
+        return f"vBinary({self.to_ical()})"
 
     def to_ical(self):
         return binascii.b2a_base64(self.obj.encode('utf-8'))[:-1]
@@ -151,9 +97,13 @@ class vBinary(object):
     def from_ical(ical):
         try:
             return base64.b64decode(ical)
-        except UnicodeError:
+        except (ValueError, UnicodeError):
             raise ValueError('Not valid base 64 encoding.')
 
+    def __eq__(self, other):
+        """self == other"""
+        return isinstance(other, vBinary) and self.obj == other.obj
+
 
 class vBoolean(int):
     """Returns specific string according to state.
@@ -161,34 +111,56 @@ class vBoolean(int):
     BOOL_MAP = CaselessDict({'true': True, 'false': False})
 
     def __new__(cls, *args, **kwargs):
-        self = super(vBoolean, cls).__new__(cls, *args, **kwargs)
+        self = super().__new__(cls, *args, **kwargs)
         self.params = Parameters()
         return self
 
     def to_ical(self):
-        if self:
-            return b'TRUE'
-        return b'FALSE'
+        return b'TRUE' if self else b'FALSE'
 
     @classmethod
     def from_ical(cls, ical):
         try:
             return cls.BOOL_MAP[ical]
-        except:
-            raise ValueError("Expected 'TRUE' or 'FALSE'. Got %s" % ical)
+        except Exception:
+            raise ValueError(f"Expected 'TRUE' or 'FALSE'. Got {ical}")
 
 
-class vCalAddress(compat.unicode_type):
-    """This just returns an unquoted string.
+class vText(str):
+    """Simple text.
     """
+
     def __new__(cls, value, encoding=DEFAULT_ENCODING):
         value = to_unicode(value, encoding=encoding)
-        self = super(vCalAddress, cls).__new__(cls, value)
+        self = super().__new__(cls, value)
+        self.encoding = encoding
+        self.params = Parameters()
+        return self
+
+    def __repr__(self) -> str:
+        return f"vText({self.to_ical()!r})"
+
+    def to_ical(self) -> bytes:
+        return escape_char(self).encode(self.encoding)
+
+    @classmethod
+    def from_ical(cls, ical:ICAL_TYPE):
+        ical_unesc = unescape_char(ical)
+        return cls(ical_unesc)
+
+
+class vCalAddress(str):
+    """This just returns an unquoted string.
+    """
+
+    def __new__(cls, value, encoding=DEFAULT_ENCODING):
+        value = to_unicode(value, encoding=encoding)
+        self = super().__new__(cls, value)
         self.params = Parameters()
         return self
 
     def __repr__(self):
-        return "vCalAddress('%s')" % self.to_ical()
+        return f"vCalAddress('{self}')"
 
     def to_ical(self):
         return self.encode(DEFAULT_ENCODING)
@@ -201,44 +173,47 @@ class vCalAddress(compat.unicode_type):
 class vFloat(float):
     """Just a float.
     """
+
     def __new__(cls, *args, **kwargs):
-        self = super(vFloat, cls).__new__(cls, *args, **kwargs)
+        self = super().__new__(cls, *args, **kwargs)
         self.params = Parameters()
         return self
 
     def to_ical(self):
-        return compat.unicode_type(self).encode('utf-8')
+        return str(self).encode('utf-8')
 
     @classmethod
     def from_ical(cls, ical):
         try:
             return cls(ical)
-        except:
-            raise ValueError('Expected float value, got: %s' % ical)
+        except Exception:
+            raise ValueError(f'Expected float value, got: {ical}')
 
 
 class vInt(int):
     """Just an int.
     """
+
     def __new__(cls, *args, **kwargs):
-        self = super(vInt, cls).__new__(cls, *args, **kwargs)
+        self = super().__new__(cls, *args, **kwargs)
         self.params = Parameters()
         return self
 
-    def to_ical(self):
-        return compat.unicode_type(self).encode('utf-8')
+    def to_ical(self) -> bytes:
+        return str(self).encode('utf-8')
 
     @classmethod
-    def from_ical(cls, ical):
+    def from_ical(cls, ical:ICAL_TYPE):
         try:
             return cls(ical)
-        except:
-            raise ValueError('Expected int, got: %s' % ical)
+        except Exception:
+            raise ValueError(f'Expected int, got: {ical}')
 
 
-class vDDDLists(object):
+class vDDDLists:
     """A list of vDDDTypes values.
     """
+
     def __init__(self, dt_list):
         if not hasattr(dt_list, '__iter__'):
             dt_list = [dt_list]
@@ -256,7 +231,7 @@ class vDDDLists(object):
         self.dts = vDDD
 
     def to_ical(self):
-        dts_ical = (dt.to_ical() for dt in self.dts)
+        dts_ical = (from_unicode(dt.to_ical()) for dt in self.dts)
         return b",".join(dts_ical)
 
     @staticmethod
@@ -267,33 +242,62 @@ class vDDDLists(object):
             out.append(vDDDTypes.from_ical(ical_dt, timezone=timezone))
         return out
 
-class vCategory(object):
+    def __eq__(self, other):
+        if not isinstance(other, vDDDLists):
+            return False
+        return self.dts == other.dts
+
+
+class vCategory:
 
     def __init__(self, c_list):
-        if not hasattr(c_list, '__iter__'):
-            d_list = [c_list]
+        if not hasattr(c_list, '__iter__') or isinstance(c_list, str):
+            c_list = [c_list]
         self.cats = [vText(c) for c in c_list]
+        self.params = Parameters()
+
+    def __iter__(self):
+        return iter(vCategory.from_ical(self.to_ical()))
 
     def to_ical(self):
         return b",".join([c.to_ical() for c in self.cats])
 
     @staticmethod
-    def from_ical(ical, timezone=None):
-        out = unescape_char(ical).split(",")
+    def from_ical(ical):
+        ical = to_unicode(ical)
+        out = unescape_char(ical).split(',')
         return out
 
+    def __eq__(self, other):
+        """self == other"""
+        return isinstance(other, vCategory) and self.cats == other.cats
 
-class vDDDTypes(object):
+
+class TimeBase:
+    """Make classes with a datetime/date comparable."""
+
+    def __eq__(self, other):
+        """self == other"""
+        if isinstance(other, TimeBase):
+            return self.params == other.params and self.dt == other.dt
+        return False
+
+    def __hash__(self):
+        return hash(self.dt)
+
+
+class vDDDTypes(TimeBase):
     """A combined Datetime, Date or Duration parser/generator. Their format
     cannot be confused, and often values can be of either types.
     So this is practical.
     """
+
     def __init__(self, dt):
         if not isinstance(dt, (datetime, date, timedelta, time, tuple)):
             raise ValueError('You must use datetime, date, timedelta, '
                              'time or tuple (for periods)')
-        if isinstance(dt, datetime):
-            self.params = Parameters({'value': 'DATE-TIME'})
+        if isinstance(dt, (datetime, timedelta)):
+            self.params = Parameters()
         elif isinstance(dt, date):
             self.params = Parameters({'value': 'DATE'})
         elif isinstance(dt, time):
@@ -301,15 +305,10 @@ class vDDDTypes(object):
         elif isinstance(dt, tuple):
             self.params = Parameters({'value': 'PERIOD'})
 
-        if (isinstance(dt, datetime) or isinstance(dt, time))\
-                and getattr(dt, 'tzinfo', False):
-            tzinfo = dt.tzinfo
-            if tzinfo is not pytz.utc and\
-               (tzutc is None or not isinstance(tzinfo, tzutc)):
-                # set the timezone as a parameter to the property
-                tzid = tzid_from_dt(dt)
-                if tzid:
-                    self.params.update({'TZID': tzid})
+        tzid = tzid_from_dt(dt) if isinstance(dt, (datetime, time)) else None
+        if tzid is not None and tzid != 'UTC':
+            self.params.update({'TZID': tzid})
+
         self.dt = dt
 
     def to_ical(self):
@@ -325,7 +324,7 @@ class vDDDTypes(object):
         elif isinstance(dt, tuple) and len(dt) == 2:
             return vPeriod(dt).to_ical()
         else:
-            raise ValueError('Unknown date type: {}'.format(type(dt)))
+            raise ValueError(f'Unknown date type: {type(dt)}')
 
     @classmethod
     def from_ical(cls, ical, timezone=None):
@@ -335,7 +334,7 @@ class vDDDTypes(object):
         if u.startswith(('P', '-P', '+P')):
             return vDuration.from_ical(ical)
         if '/' in u:
-            return vPeriod.from_ical(ical)
+            return vPeriod.from_ical(ical, timezone=timezone)
 
         if len(ical) in (15, 16):
             return vDatetime.from_ical(ical, timezone=timezone)
@@ -345,13 +344,18 @@ class vDDDTypes(object):
             return vTime.from_ical(ical)
         else:
             raise ValueError(
-                "Expected datetime, date, or time, got: '%s'" % ical
+                f"Expected datetime, date, or time, got: '{ical}'"
             )
 
+    def __repr__(self):
+        """repr(self)"""
+        return f"{self.__class__.__name__}({self.dt}, {self.params})"
 
-class vDate(object):
+
+class vDate(TimeBase):
     """Render and generates iCalendar date format.
     """
+
     def __init__(self, dt):
         if not isinstance(dt, date):
             raise ValueError('Value MUST be a date instance')
@@ -359,7 +363,7 @@ class vDate(object):
         self.params = Parameters({'value': 'DATE'})
 
     def to_ical(self):
-        s = "%04d%02d%02d" % (self.dt.year, self.dt.month, self.dt.day)
+        s = f"{self.dt.year:04}{self.dt.month:02}{self.dt.day:02}"
         return s.encode('utf-8')
 
     @staticmethod
@@ -371,21 +375,22 @@ class vDate(object):
                 int(ical[6:8]),  # day
             )
             return date(*timetuple)
-        except:
-            raise ValueError('Wrong date format %s' % ical)
+        except Exception:
+            raise ValueError(f'Wrong date format {ical}')
 
 
-class vDatetime(object):
+class vDatetime(TimeBase):
     """Render and generates icalendar datetime format.
 
-    vDatetime is timezone aware and uses the pytz library, an implementation of
-    the Olson database in Python. When a vDatetime object is created from an
-    ical string, you can pass a valid pytz timezone identifier. When a
+    vDatetime is timezone aware and uses a timezone library.
+    When a vDatetime object is created from an
+    ical string, you can pass a valid timezone identifier. When a
     vDatetime object is created from a python datetime object, it uses the
     tzinfo component, if present. Otherwise an timezone-naive object is
     created. Be aware that there are certain limitations with timezone naive
     DATE-TIME components in the icalendar standard.
     """
+
     def __init__(self, dt):
         self.dt = dt
         self.params = Parameters()
@@ -394,14 +399,7 @@ class vDatetime(object):
         dt = self.dt
         tzid = tzid_from_dt(dt)
 
-        s = "%04d%02d%02dT%02d%02d%02d" % (
-            dt.year,
-            dt.month,
-            dt.day,
-            dt.hour,
-            dt.minute,
-            dt.second
-        )
+        s = f"{dt.year:04}{dt.month:02}{dt.day:02}T{dt.hour:02}{dt.minute:02}{dt.second:02}"
         if tzid == 'UTC':
             s += "Z"
         elif tzid:
@@ -412,13 +410,7 @@ class vDatetime(object):
     def from_ical(ical, timezone=None):
         tzinfo = None
         if timezone:
-            try:
-                tzinfo = pytz.timezone(timezone)
-            except pytz.UnknownTimeZoneError:
-                if timezone in WINDOWS_TO_OLSON:
-                    tzinfo = pytz.timezone(WINDOWS_TO_OLSON.get(timezone))
-                else:
-                    tzinfo = _timezone_cache.get(timezone, None)
+            tzinfo = tzp.timezone(timezone)
 
         try:
             timetuple = (
@@ -430,18 +422,18 @@ class vDatetime(object):
                 int(ical[13:15]),  # second
             )
             if tzinfo:
-                return tzinfo.localize(datetime(*timetuple))
+                return tzp.localize(datetime(*timetuple), tzinfo)
             elif not ical[15:]:
                 return datetime(*timetuple)
             elif ical[15:16] == 'Z':
-                return pytz.utc.localize(datetime(*timetuple))
+                return tzp.localize_utc(datetime(*timetuple))
             else:
                 raise ValueError(ical)
-        except:
-            raise ValueError('Wrong datetime format: %s' % ical)
+        except Exception:
+            raise ValueError(f'Wrong datetime format: {ical}')
 
 
-class vDuration(object):
+class vDuration(TimeBase):
     """Subclass of timedelta that renders itself in the iCalendar DURATION
     format.
     """
@@ -454,58 +446,67 @@ class vDuration(object):
 
     def to_ical(self):
         sign = ""
-        if self.td.days < 0:
+        td = self.td
+        if td.days < 0:
             sign = "-"
-            self.td = -self.td
+            td = -td
         timepart = ""
-        if self.td.seconds:
+        if td.seconds:
             timepart = "T"
-            hours = self.td.seconds // 3600
-            minutes = self.td.seconds % 3600 // 60
-            seconds = self.td.seconds % 60
+            hours = td.seconds // 3600
+            minutes = td.seconds % 3600 // 60
+            seconds = td.seconds % 60
             if hours:
-                timepart += "%dH" % hours
+                timepart += f"{hours}H"
             if minutes or (hours and seconds):
-                timepart += "%dM" % minutes
+                timepart += f"{minutes}M"
             if seconds:
-                timepart += "%dS" % seconds
-        if self.td.days == 0 and timepart:
-            return (compat.unicode_type(sign).encode('utf-8') + b'P' +
-                    compat.unicode_type(timepart).encode('utf-8'))
+                timepart += f"{seconds}S"
+        if td.days == 0 and timepart:
+            return (str(sign).encode('utf-8') + b'P'
+                    + str(timepart).encode('utf-8'))
         else:
-            return (compat.unicode_type(sign).encode('utf-8') + b'P' +
-                    compat.unicode_type(abs(self.td.days)).encode('utf-8') +
-                    b'D' + compat.unicode_type(timepart).encode('utf-8'))
+            return (str(sign).encode('utf-8') + b'P'
+                    + str(abs(td.days)).encode('utf-8')
+                    + b'D' + str(timepart).encode('utf-8'))
 
     @staticmethod
     def from_ical(ical):
-        try:
-            match = DURATION_REGEX.match(ical)
-            sign, weeks, days, hours, minutes, seconds = match.groups()
-            if weeks:
-                value = timedelta(weeks=int(weeks))
-            else:
-                value = timedelta(days=int(days or 0),
-                                  hours=int(hours or 0),
-                                  minutes=int(minutes or 0),
-                                  seconds=int(seconds or 0))
-            if sign == '-':
-                value = -value
-            return value
-        except:
-            raise ValueError('Invalid iCalendar duration: %s' % ical)
+        match = DURATION_REGEX.match(ical)
+        if not match:
+            raise ValueError(f'Invalid iCalendar duration: {ical}')
+
+        sign, weeks, days, hours, minutes, seconds = match.groups()
+        value = timedelta(
+            weeks=int(weeks or 0),
+            days=int(days or 0),
+            hours=int(hours or 0),
+            minutes=int(minutes or 0),
+            seconds=int(seconds or 0)
+        )
+
+        if sign == '-':
+            value = -value
+
+        return value
+
+    @property
+    def dt(self):
+        """The time delta for compatibility."""
+        return self.td
 
 
-class vPeriod(object):
+class vPeriod(TimeBase):
     """A precise period of time.
     """
+
     def __init__(self, per):
         start, end_or_duration = per
         if not (isinstance(start, datetime) or isinstance(start, date)):
             raise ValueError('Start value MUST be a datetime or date instance')
-        if not (isinstance(end_or_duration, datetime) or
-                isinstance(end_or_duration, date) or
-                isinstance(end_or_duration, timedelta)):
+        if not (isinstance(end_or_duration, datetime)
+                or isinstance(end_or_duration, date)
+                or isinstance(end_or_duration, timedelta)):
             raise ValueError('end_or_duration MUST be a datetime, '
                              'date or timedelta instance')
         by_duration = 0
@@ -519,7 +520,7 @@ class vPeriod(object):
         if start > end:
             raise ValueError("Start time is greater than end time")
 
-        self.params = Parameters()
+        self.params = Parameters({'value': 'PERIOD'})
         # set the timezone identifier
         # does not support different timezones for start and end
         tzid = tzid_from_dt(start)
@@ -531,11 +532,6 @@ class vPeriod(object):
         self.by_duration = by_duration
         self.duration = duration
 
-    def __cmp__(self, other):
-        if not isinstance(other, vPeriod):
-            raise NotImplementedError('Cannot compare vPeriod with %r' % other)
-        return cmp((self.start, self.end), (other.start, other.end))
-
     def overlaps(self, other):
         if self.start > other.start:
             return other.overlaps(self)
@@ -545,30 +541,35 @@ class vPeriod(object):
 
     def to_ical(self):
         if self.by_duration:
-            return (vDatetime(self.start).to_ical() + b'/' +
-                    vDuration(self.duration).to_ical())
-        return (vDatetime(self.start).to_ical() + b'/' +
-                vDatetime(self.end).to_ical())
+            return (vDatetime(self.start).to_ical() + b'/'
+                    + vDuration(self.duration).to_ical())
+        return (vDatetime(self.start).to_ical() + b'/'
+                + vDatetime(self.end).to_ical())
 
     @staticmethod
-    def from_ical(ical):
+    def from_ical(ical, timezone=None):
         try:
             start, end_or_duration = ical.split('/')
-            start = vDDDTypes.from_ical(start)
-            end_or_duration = vDDDTypes.from_ical(end_or_duration)
+            start = vDDDTypes.from_ical(start, timezone=timezone)
+            end_or_duration = vDDDTypes.from_ical(end_or_duration, timezone=timezone)
             return (start, end_or_duration)
-        except:
-            raise ValueError('Expected period format, got: %s' % ical)
+        except Exception:
+            raise ValueError(f'Expected period format, got: {ical}')
 
     def __repr__(self):
         if self.by_duration:
             p = (self.start, self.duration)
         else:
             p = (self.start, self.end)
-        return 'vPeriod(%r)' % (p, )
+        return f'vPeriod({p!r})'
+
+    @property
+    def dt(self):
+        """Make this cooperate with the other vDDDTypes."""
+        return (self.start, (self.duration if self.by_duration else self.end))
 
 
-class vWeekday(compat.unicode_type):
+class vWeekday(str):
     """This returns an unquoted weekday abbrevation.
     """
     week_days = CaselessDict({
@@ -577,16 +578,16 @@ class vWeekday(compat.unicode_type):
 
     def __new__(cls, value, encoding=DEFAULT_ENCODING):
         value = to_unicode(value, encoding=encoding)
-        self = super(vWeekday, cls).__new__(cls, value)
+        self = super().__new__(cls, value)
         match = WEEKDAY_RULE.match(self)
         if match is None:
-            raise ValueError('Expected weekday abbrevation, got: %s' % self)
+            raise ValueError(f'Expected weekday abbrevation, got: {self}')
         match = match.groupdict()
         sign = match['signal']
         weekday = match['weekday']
         relative = match['relative']
         if weekday not in vWeekday.week_days or sign not in '+-':
-            raise ValueError('Expected weekday abbrevation, got: %s' % self)
+            raise ValueError(f'Expected weekday abbrevation, got: {self}')
         self.relative = relative and int(relative) or None
         self.params = Parameters()
         return self
@@ -598,11 +599,11 @@ class vWeekday(compat.unicode_type):
     def from_ical(cls, ical):
         try:
             return cls(ical.upper())
-        except:
-            raise ValueError('Expected weekday abbrevation, got: %s' % ical)
+        except Exception:
+            raise ValueError(f'Expected weekday abbrevation, got: {ical}')
 
 
-class vFrequency(compat.unicode_type):
+class vFrequency(str):
     """A simple class that catches illegal values.
     """
 
@@ -618,9 +619,9 @@ class vFrequency(compat.unicode_type):
 
     def __new__(cls, value, encoding=DEFAULT_ENCODING):
         value = to_unicode(value, encoding=encoding)
-        self = super(vFrequency, cls).__new__(cls, value)
+        self = super().__new__(cls, value)
         if self not in vFrequency.frequencies:
-            raise ValueError('Expected frequency, got: %s' % self)
+            raise ValueError(f'Expected frequency, got: {self}')
         self.params = Parameters()
         return self
 
@@ -631,8 +632,95 @@ class vFrequency(compat.unicode_type):
     def from_ical(cls, ical):
         try:
             return cls(ical.upper())
-        except:
-            raise ValueError('Expected frequency, got: %s' % ical)
+        except Exception:
+            raise ValueError(f'Expected frequency, got: {ical}')
+
+
+class vMonth(int):
+    """The number of the month for recurrence.
+
+    In :rfc:`5545`, this is just an int.
+    In :rfc:`7529`, this can be followed by `L` to indicate a leap month.
+
+        >>> from icalendar import vMonth
+        >>> vMonth(1) # first month January
+        vMonth('1')
+        >>> vMonth("5L") # leap month in Hebrew calendar
+        vMonth('5L')
+        >>> vMonth(1).leap
+        False
+        >>> vMonth("5L").leap
+        True
+
+    Definition from RFC::
+
+        type-bymonth = element bymonth {
+           xsd:positiveInteger |
+           xsd:string
+        }
+    """
+    def __new__(cls, month:Union[str, int]):
+        if isinstance(month, vMonth):
+            return cls(month.to_ical().decode())
+        if isinstance(month, str):
+            if month.isdigit():
+                month_index = int(month)
+                leap = False
+            else:
+                if not month[-1] == "L" and month[:-1].isdigit():
+                    raise ValueError(f"Invalid month: {month!r}")
+                month_index = int(month[:-1])
+                leap = True
+        else:
+            leap = False
+            month_index = int(month)
+        self = super().__new__(cls, month_index)
+        self.leap = leap
+        self.params = Parameters()
+        return self
+
+    def to_ical(self) -> bytes:
+        """The ical representation."""
+        return str(self).encode('utf-8')
+
+    @classmethod
+    def from_ical(cls, ical: str):
+        return cls(ical)
+
+    def leap():
+        doc = "Whether this is a leap month."
+        def fget(self) -> bool:
+            return self._leap
+        def fset(self, value:bool) -> None:
+            self._leap = value
+        return locals()
+    leap = property(**leap())
+
+
+    def __repr__(self) -> str:
+        """repr(self)"""
+        return f"{self.__class__.__name__}({str(self)!r})"
+
+    def __str__(self) -> str:
+        """str(self)"""
+        return f"{int(self)}{'L' if self.leap else ''}"
+
+
+class vSkip(vText, Enum):
+    """Skip values for RRULE.
+
+    These are defined in :rfc:`7529`.
+
+    OMIT  is the default value.
+    """
+
+    OMIT = "OMIT"
+    FORWARD = "FORWARD"
+    BACKWARD = "BACKWARD"
+
+    def __reduce_ex__(self, _p):
+        """For pickling."""
+        return self.__class__, (self._name_,)
 
 
 class vRecur(CaselessDict):
@@ -644,10 +732,10 @@ class vRecur(CaselessDict):
 
     # Mac iCal ignores RRULEs where FREQ is not the first rule part.
     # Sorts parts according to the order listed in RFC 5545, section 3.3.10.
-    canonical_order = ("FREQ", "UNTIL", "COUNT", "INTERVAL",
-                       "BYSECOND", "BYMINUTE", "BYHOUR", "BYDAY",
+    canonical_order = ("RSCALE", "FREQ", "UNTIL", "COUNT", "INTERVAL",
+                       "BYSECOND", "BYMINUTE", "BYHOUR", "BYDAY", "BYWEEKDAY",
                        "BYMONTHDAY", "BYYEARDAY", "BYWEEKNO", "BYMONTH",
-                       "BYSETPOS", "WKST")
+                       "BYSETPOS", "WKST", "SKIP")
 
     types = CaselessDict({
         'COUNT': vInt,
@@ -658,22 +746,27 @@ class vRecur(CaselessDict):
         'BYWEEKNO': vInt,
         'BYMONTHDAY': vInt,
         'BYYEARDAY': vInt,
-        'BYMONTH': vInt,
+        'BYMONTH': vMonth,
         'UNTIL': vDDDTypes,
         'BYSETPOS': vInt,
         'WKST': vWeekday,
         'BYDAY': vWeekday,
         'FREQ': vFrequency,
+        'BYWEEKDAY': vWeekday,
+        'SKIP': vSkip,
     })
 
     def __init__(self, *args, **kwargs):
-        super(vRecur, self).__init__(*args, **kwargs)
+        for k, v in kwargs.items():
+            if not isinstance(v, SEQUENCE_TYPES):
+                kwargs[k] = [v]
+        super().__init__(*args, **kwargs)
         self.params = Parameters()
 
     def to_ical(self):
         result = []
         for key, vals in self.sorted_items():
-            typ = self.types[key]
+            typ = self.types.get(key, vText)
             if not isinstance(vals, SEQUENCE_TYPES):
                 vals = [vals]
             vals = b','.join(typ(val).to_ical() for val in vals)
@@ -691,7 +784,7 @@ class vRecur(CaselessDict):
         return [parser.from_ical(v) for v in values.split(',')]
 
     @classmethod
-    def from_ical(cls, ical):
+    def from_ical(cls, ical: str):
         if isinstance(ical, cls):
             return ical
         try:
@@ -704,42 +797,21 @@ class vRecur(CaselessDict):
                     # FREQ=YEARLY;BYMONTH=11;BYDAY=1SU;
                     continue
                 recur[key] = cls.parse_type(key, vals)
-            return dict(recur)
+            return cls(recur)
+        except ValueError:
+            raise
         except:
-            raise ValueError('Error in recurrence rule: %s' % ical)
+            raise ValueError(f'Error in recurrence rule: {ical}')
 
 
-class vText(compat.unicode_type):
-    """Simple text.
-    """
-
-    def __new__(cls, value, encoding=DEFAULT_ENCODING):
-        value = to_unicode(value, encoding=encoding)
-        self = super(vText, cls).__new__(cls, value)
-        self.encoding = encoding
-        self.params = Parameters()
-        return self
-
-    def __repr__(self):
-        return "vText('%s')" % self.to_ical()
-
-    def to_ical(self):
-        return escape_char(self).encode(self.encoding)
-
-    @classmethod
-    def from_ical(cls, ical):
-        ical_unesc = unescape_char(ical)
-        return cls(ical_unesc)
-
-
-class vTime(object):
+class vTime(TimeBase):
     """Render and generates iCalendar time format.
     """
 
     def __init__(self, *args):
         if len(args) == 1:
             if not isinstance(args[0], (time, datetime)):
-                raise ValueError('Expected a datetime.time, got: %s' % args[0])
+                raise ValueError(f'Expected a datetime.time, got: {args[0]}')
             self.dt = args[0]
         else:
             self.dt = time(*args)
@@ -754,17 +826,17 @@ class vTime(object):
         try:
             timetuple = (int(ical[:2]), int(ical[2:4]), int(ical[4:6]))
             return time(*timetuple)
-        except:
-            raise ValueError('Expected time, got: %s' % ical)
+        except Exception:
+            raise ValueError(f'Expected time, got: {ical}')
 
 
-class vUri(compat.unicode_type):
+class vUri(str):
     """Uniform resource identifier is basically just an unquoted string.
     """
 
     def __new__(cls, value, encoding=DEFAULT_ENCODING):
         value = to_unicode(value, encoding=encoding)
-        self = super(vUri, cls).__new__(cls, value)
+        self = super().__new__(cls, value)
         self.params = Parameters()
         return self
 
@@ -775,11 +847,11 @@ class vUri(compat.unicode_type):
     def from_ical(cls, ical):
         try:
             return cls(ical)
-        except:
-            raise ValueError('Expected , got: %s' % ical)
+        except Exception:
+            raise ValueError(f'Expected , got: {ical}')
 
 
-class vGeo(object):
+class vGeo:
     """A special type that is only indirectly defined in the rfc.
     """
 
@@ -788,7 +860,7 @@ class vGeo(object):
             latitude, longitude = (geo[0], geo[1])
             latitude = float(latitude)
             longitude = float(longitude)
-        except:
+        except Exception:
             raise ValueError('Input must be (float, float) for '
                              'latitude and longitude')
         self.latitude = latitude
@@ -796,25 +868,29 @@ class vGeo(object):
         self.params = Parameters()
 
     def to_ical(self):
-        return '%s;%s' % (self.latitude, self.longitude)
+        return f'{self.latitude};{self.longitude}'
 
     @staticmethod
     def from_ical(ical):
         try:
             latitude, longitude = ical.split(';')
             return (float(latitude), float(longitude))
-        except:
-            raise ValueError("Expected 'float;float' , got: %s" % ical)
+        except Exception:
+            raise ValueError(f"Expected 'float;float' , got: {ical}")
+
+    def __eq__(self, other):
+        return self.to_ical() == other.to_ical()
 
 
-class vUTCOffset(object):
+class vUTCOffset:
     """Renders itself as a utc offset.
     """
 
-    ignore_exceptions = False   # if True, and we cannot parse this
-                                # component, we will silently ignore
-                                # it, rather than let the exception
-                                # propagate upwards
+    ignore_exceptions = False  # if True, and we cannot parse this
+
+    # component, we will silently ignore
+    # it, rather than let the exception
+    # propagate upwards
 
     def __init__(self, td):
         if not isinstance(td, timedelta):
@@ -838,9 +914,9 @@ class vUTCOffset(object):
         minutes = abs((seconds % 3600) // 60)
         seconds = abs(seconds % 60)
         if seconds:
-            duration = '%02i%02i%02i' % (hours, minutes, seconds)
+            duration = f'{hours:02}{minutes:02}{seconds:02}'
         else:
-            duration = '%02i%02i' % (hours, minutes)
+            duration = f'{hours:02}{minutes:02}'
         return sign % duration
 
     @classmethod
@@ -853,24 +929,30 @@ class vUTCOffset(object):
                                              int(ical[3:5]),
                                              int(ical[5:7] or 0))
             offset = timedelta(hours=hours, minutes=minutes, seconds=seconds)
-        except:
-            raise ValueError('Expected utc offset, got: %s' % ical)
+        except Exception:
+            raise ValueError(f'Expected utc offset, got: {ical}')
         if not cls.ignore_exceptions and offset >= timedelta(hours=24):
             raise ValueError(
-                'Offset must be less than 24 hours, was %s' % ical)
+                f'Offset must be less than 24 hours, was {ical}')
         if sign == '-':
             return -offset
         return offset
 
+    def __eq__(self, other):
+        if not isinstance(other, vUTCOffset):
+            return False
+        return self.td == other.td
 
-class vInline(compat.unicode_type):
+
+class vInline(str):
     """This is an especially dumb class that just holds raw unparsed text and
     has parameters. Conversion of inline values are handled by the Component
     class, so no further processing is needed.
     """
+
     def __new__(cls, value, encoding=DEFAULT_ENCODING):
         value = to_unicode(value, encoding=encoding)
-        self = super(vInline, cls).__new__(cls, value)
+        self = super().__new__(cls, value)
         self.params = Parameters()
         return self
 
@@ -883,7 +965,7 @@ class vInline(compat.unicode_type):
 
 
 class TypesFactory(CaselessDict):
-    """All Value types defined in rfc 2445 are registered in this factory
+    """All Value types defined in RFC 5545 are registered in this factory
     class.
 
     The value and parameter names don't overlap. So one factory is enough for
@@ -892,7 +974,7 @@ class TypesFactory(CaselessDict):
 
     def __init__(self, *args, **kwargs):
         "Set keys to upper for initial dict"
-        super(TypesFactory, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.all_types = (
             vBinary,
             vBoolean,
@@ -1041,3 +1123,11 @@ class TypesFactory(CaselessDict):
         type_class = self.for_property(name)
         decoded = type_class.from_ical(value)
         return decoded
+
+
+__all__ = ["DURATION_REGEX", "TimeBase", "TypesFactory", "WEEKDAY_RULE",
+           "tzid_from_dt", "vBinary", "vBoolean", "vCalAddress",
+           "vCategory", "vDDDLists", "vDDDTypes", "vDate", "vDatetime",
+           "vDuration", "vFloat", "vFrequency", "vGeo", "vInline", "vInt",
+           "vMonth", "vPeriod", "vRecur", "vSkip", "vText", "vTime",
+           "vUTCOffset", "vUri", "vWeekday"]
